package com.tearabite.base;

import com.pi4j.util.Console;

import java.time.Duration;

public abstract class Component {
    protected static Console console = new Console();

    protected Component() {
    }

    /**
     * Override this method to clean up all used resources
     */
    public void reset() {
        //nothing to do by default
    }

    protected void logInfo(String msg, Object... args) {
        console.println(String.format(msg, args));
    }

    protected void logError(String msg, Object... args) {
        console.println(String.format(msg, args));
    }

    protected void logDebug(String msg, Object... args) {
        console.println(String.format(msg, args));
    }

    protected void logException(String msg, Throwable exception){
        console.println(String.format(msg, exception.getMessage()));
    }

    /**
     * Utility function to sleep for the specified amount of milliseconds.
     * An {@link InterruptedException} will be caught and ignored while setting the interrupt flag again.
     *
     * @param duration Time to sleep
     */
    protected void delay(Duration duration) {
        try {
            long nanos = duration.toNanos();
            long millis = nanos / 1_000_000;
            int remainingNanos = (int) (nanos % 1_000_000);
            Thread.currentThread().sleep(millis, remainingNanos);
        } catch (InterruptedException e) {
            Thread.currentThread().interrupt();
        }
    }
}